
function fifo_gen(p)


fid = fopen('../verilog/FFT_FIFO.v','w');

% -- wb_[
fprintf(fid,'`ifdef SYNTOOL_OFF \n');
fprintf(fid,'`else \n');
fprintf(fid,'`include "definition.v" \n');
fprintf(fid,'`endif \n\n');

W = p.Wordlength;

for s=1:log2(p.N)
    fifo_num = p.N / 2^s;
    fprintf(fid,'\n');
    fprintf(fid,'module FIFO_ST%d(CLK, Fin_r, Fin_i, Fout_r, Fout_i);\n', s);
    
    fprintf(fid,'input  CLK;\n');
    fprintf(fid,'input  [`W-1:0]  Fin_r, Fin_i;\n');
    fprintf(fid,'output [`W-1:0]  Fout_r, Fout_i;\n');
     
    if (fifo_num == 1)
        fprintf(fid,'reg [`W-1:0]  Fout_r, Fout_i;\n');
        fprintf(fid,'always@(posedge CLK)\n');
        fprintf(fid,'begin\n'); 
        fprintf(fid,'Fout_r <= Fin_r;\n');
        fprintf(fid,'Fout_i <= Fin_i;\n');
    elseif (fifo_num == 2)
        fprintf(fid,'reg [`W-1:0]  Fout_r, Fout_i;\n');
        fprintf(fid,'reg    [%d:0] F0;\n', (2*W)*(fifo_num-1)-1);
        fprintf(fid,'always@(posedge CLK)\n');
        fprintf(fid,'begin\n');
        fprintf(fid,'F0[%d:0] <= {Fin_r, Fin_i};\n', W*2-1);
        fprintf(fid,'Fout_r <= F0[%d:%d];\n', W*2-1, W);
        fprintf(fid,'Fout_i <= F0[%d:%d];\n', W-1, 0);  
    else
        fprintf(fid,'reg    [`W-1:0] Fout_r, Fout_i;\n');
        fprintf(fid,'reg    [`W-1:0] Fout_r0, Fout_i0;\n');
        fprintf(fid,'reg    [`W-1:0] mem_r [%d:0];\n', fifo_num-2);
        fprintf(fid,'reg    [`W-1:0] mem_i [%d:0];\n', fifo_num-2);
        fprintf(fid,'reg    [%d:0] k;\n', log2(fifo_num)-1);
        fprintf(fid,'wire   [%d:0] kr;\n', log2(fifo_num)-1);
        fprintf(fid,'wire   we;\n');
        fprintf(fid,'assign we = 1;\n');
        fprintf(fid,'assign kr = (k==%d) ? 0 : k+1;\n', fifo_num-2);
        fprintf(fid,'initial begin\n'); 
        fprintf(fid,' k=0;\n'); 
        fprintf(fid,'end\n'); 
        fprintf(fid,'always@(posedge CLK)\n');
        fprintf(fid,'begin\n'); 
        fprintf(fid,'        if (k==%d) k<=0; else k<=k+1; \n', fifo_num-2);
        fprintf(fid,'        if (we==1) begin \n');
        fprintf(fid,'            mem_r[k] = Fin_r;\n');
        fprintf(fid,'            mem_i[k] = Fin_i;\n'); 
        fprintf(fid,'        end \n');
        fprintf(fid,'        Fout_r0 <= mem_r[kr];\n');
        fprintf(fid,'        Fout_i0 <= mem_i[kr];\n');  
        fprintf(fid,'        Fout_r <= Fout_r0;\n');
        fprintf(fid,'        Fout_i <= Fout_i0;\n');          
    end

    fprintf(fid,'end\n');
    % -- tb_[
    fprintf(fid,'endmodule\n\n');
end

fclose(fid);