`ifdef SYNTOOL_OFF 
`else 
`include "definition.v" 
`endif 


module BUTTERFLY_STAGE1(CLK, k, X_r, X_i, Y_r, Y_i);
input  CLK;
input  [`N_AD_BUS-1:0]  k;
input  [`W-1:0]  X_r, X_i;
output [`W-1:0]  Y_r, Y_i;
reg    [`W-1:0]  Y_r, Y_i;
wire   [`W-1:0]  A_r, A_i, B_r, B_i;
wire   [`W-1:0]  Fin_r, Fin_i;
wire   [`W-1:0]  Fout_r, Fout_i;
wire   [`W-1:0]  Min_r,  Min_i;
wire   [`W-1:0]  Mbf_r, Mbf_i;
wire   [`W-1:0]  Mout_r, Mout_i;
wire   [`W-1:0]  W_r, W_i; 
wire   [1:0]     W_ex_r, W_ex_i;
wire             scaling_bit = 1;
reg    [`W-1:0]  Min_r_reg,  Min_i_reg;
reg    [`W-1:0]  A_r_delay1,  A_i_delay1;
reg    [`W-1:0]  A_r_delay2,  A_i_delay2;
reg    [`W-1:0]  A_r_delay3,  A_i_delay3;
reg    [`W-1:0]  A_r_delay4,  A_i_delay4;
reg              k_delay1;
reg              k_delay2;
reg              k_delay3;
reg              k_delay4;
assign Min_r = (k[12] == 0) ? Fout_r : A_r;
assign Min_i = (k[12] == 0) ? Fout_i : A_i;
assign Fin_r = (k[12] == 0) ? X_r : B_r;
assign Fin_i = (k[12] == 0) ? X_i : B_i;
assign Mbf_r = (k_delay4 == 1) ? A_r_delay4 : Mout_r;
assign Mbf_i = (k_delay4 == 1) ? A_i_delay4 : Mout_i;
FIFO_ST1 FIFO_ST1(CLK, Fin_r, Fin_i, Fout_r, Fout_i);
TWIDDLE_ST1 TWIDDLE_ST1(CLK, k[11:0], W_ex_r, W_r, W_ex_i, W_i);
CADD CADD(Fout_r, Fout_i, X_r, X_i, A_r, A_i, B_r, B_i, scaling_bit);
CMUL CMUL(CLK, Min_r_reg, Min_i_reg, W_r, W_ex_r, W_i, W_ex_i, Mout_r, Mout_i);
always@(posedge CLK)
begin
    Min_r_reg <= Min_r;  Min_i_reg <= Min_i;
    k_delay1 <= k[12];
    k_delay2 <= k_delay1;
    k_delay3 <= k_delay2;
    k_delay4 <= k_delay3;
    A_r_delay1  <= A_r;         A_i_delay1 <= A_i;
    A_r_delay2  <= A_r_delay1;  A_i_delay2 <= A_i_delay1;
    A_r_delay3  <= A_r_delay2;  A_i_delay3 <= A_i_delay2;
    A_r_delay4  <= A_r_delay3;  A_i_delay4 <= A_i_delay3;
    Y_r <= Mbf_r;
    Y_i <= Mbf_i;
end
endmodule


module BUTTERFLY_STAGE2(CLK, k, X_r, X_i, Y_r, Y_i);
input  CLK;
input  [`N_AD_BUS-1:0]  k;
input  [`W-1:0]  X_r, X_i;
output [`W-1:0]  Y_r, Y_i;
reg    [`W-1:0]  Y_r, Y_i;
wire   [`W-1:0]  A_r, A_i, B_r, B_i;
wire   [`W-1:0]  Fin_r, Fin_i;
wire   [`W-1:0]  Fout_r, Fout_i;
wire   [`W-1:0]  Min_r,  Min_i;
wire   [`W-1:0]  Mbf_r, Mbf_i;
wire   [`W-1:0]  Mout_r, Mout_i;
wire   [`W-1:0]  W_r, W_i; 
wire   [1:0]     W_ex_r, W_ex_i;
wire             scaling_bit = 1;
reg    [`W-1:0]  Min_r_reg,  Min_i_reg;
reg    [`W-1:0]  A_r_delay1,  A_i_delay1;
reg    [`W-1:0]  A_r_delay2,  A_i_delay2;
reg    [`W-1:0]  A_r_delay3,  A_i_delay3;
reg    [`W-1:0]  A_r_delay4,  A_i_delay4;
reg              k_delay1;
reg              k_delay2;
reg              k_delay3;
reg              k_delay4;
assign Min_r = (k[11] == 0) ? Fout_r : A_r;
assign Min_i = (k[11] == 0) ? Fout_i : A_i;
assign Fin_r = (k[11] == 0) ? X_r : B_r;
assign Fin_i = (k[11] == 0) ? X_i : B_i;
assign Mbf_r = (k_delay4 == 1) ? A_r_delay4 : Mout_r;
assign Mbf_i = (k_delay4 == 1) ? A_i_delay4 : Mout_i;
FIFO_ST2 FIFO_ST2(CLK, Fin_r, Fin_i, Fout_r, Fout_i);
TWIDDLE_ST2 TWIDDLE_ST2(CLK, k[10:0], W_ex_r, W_r, W_ex_i, W_i);
CADD CADD(Fout_r, Fout_i, X_r, X_i, A_r, A_i, B_r, B_i, scaling_bit);
CMUL CMUL(CLK, Min_r_reg, Min_i_reg, W_r, W_ex_r, W_i, W_ex_i, Mout_r, Mout_i);
always@(posedge CLK)
begin
    Min_r_reg <= Min_r;  Min_i_reg <= Min_i;
    k_delay1 <= k[11];
    k_delay2 <= k_delay1;
    k_delay3 <= k_delay2;
    k_delay4 <= k_delay3;
    A_r_delay1  <= A_r;         A_i_delay1 <= A_i;
    A_r_delay2  <= A_r_delay1;  A_i_delay2 <= A_i_delay1;
    A_r_delay3  <= A_r_delay2;  A_i_delay3 <= A_i_delay2;
    A_r_delay4  <= A_r_delay3;  A_i_delay4 <= A_i_delay3;
    Y_r <= Mbf_r;
    Y_i <= Mbf_i;
end
endmodule


module BUTTERFLY_STAGE3(CLK, k, X_r, X_i, Y_r, Y_i);
input  CLK;
input  [`N_AD_BUS-1:0]  k;
input  [`W-1:0]  X_r, X_i;
output [`W-1:0]  Y_r, Y_i;
reg    [`W-1:0]  Y_r, Y_i;
wire   [`W-1:0]  A_r, A_i, B_r, B_i;
wire   [`W-1:0]  Fin_r, Fin_i;
wire   [`W-1:0]  Fout_r, Fout_i;
wire   [`W-1:0]  Min_r,  Min_i;
wire   [`W-1:0]  Mbf_r, Mbf_i;
wire   [`W-1:0]  Mout_r, Mout_i;
wire   [`W-1:0]  W_r, W_i; 
wire   [1:0]     W_ex_r, W_ex_i;
wire             scaling_bit = 1;
reg    [`W-1:0]  Min_r_reg,  Min_i_reg;
reg    [`W-1:0]  A_r_delay1,  A_i_delay1;
reg    [`W-1:0]  A_r_delay2,  A_i_delay2;
reg    [`W-1:0]  A_r_delay3,  A_i_delay3;
reg    [`W-1:0]  A_r_delay4,  A_i_delay4;
reg              k_delay1;
reg              k_delay2;
reg              k_delay3;
reg              k_delay4;
assign Min_r = (k[10] == 0) ? Fout_r : A_r;
assign Min_i = (k[10] == 0) ? Fout_i : A_i;
assign Fin_r = (k[10] == 0) ? X_r : B_r;
assign Fin_i = (k[10] == 0) ? X_i : B_i;
assign Mbf_r = (k_delay4 == 1) ? A_r_delay4 : Mout_r;
assign Mbf_i = (k_delay4 == 1) ? A_i_delay4 : Mout_i;
FIFO_ST3 FIFO_ST3(CLK, Fin_r, Fin_i, Fout_r, Fout_i);
TWIDDLE_ST3 TWIDDLE_ST3(CLK, k[9:0], W_ex_r, W_r, W_ex_i, W_i);
CADD CADD(Fout_r, Fout_i, X_r, X_i, A_r, A_i, B_r, B_i, scaling_bit);
CMUL CMUL(CLK, Min_r_reg, Min_i_reg, W_r, W_ex_r, W_i, W_ex_i, Mout_r, Mout_i);
always@(posedge CLK)
begin
    Min_r_reg <= Min_r;  Min_i_reg <= Min_i;
    k_delay1 <= k[10];
    k_delay2 <= k_delay1;
    k_delay3 <= k_delay2;
    k_delay4 <= k_delay3;
    A_r_delay1  <= A_r;         A_i_delay1 <= A_i;
    A_r_delay2  <= A_r_delay1;  A_i_delay2 <= A_i_delay1;
    A_r_delay3  <= A_r_delay2;  A_i_delay3 <= A_i_delay2;
    A_r_delay4  <= A_r_delay3;  A_i_delay4 <= A_i_delay3;
    Y_r <= Mbf_r;
    Y_i <= Mbf_i;
end
endmodule


module BUTTERFLY_STAGE4(CLK, k, X_r, X_i, Y_r, Y_i);
input  CLK;
input  [`N_AD_BUS-1:0]  k;
input  [`W-1:0]  X_r, X_i;
output [`W-1:0]  Y_r, Y_i;
reg    [`W-1:0]  Y_r, Y_i;
wire   [`W-1:0]  A_r, A_i, B_r, B_i;
wire   [`W-1:0]  Fin_r, Fin_i;
wire   [`W-1:0]  Fout_r, Fout_i;
wire   [`W-1:0]  Min_r,  Min_i;
wire   [`W-1:0]  Mbf_r, Mbf_i;
wire   [`W-1:0]  Mout_r, Mout_i;
wire   [`W-1:0]  W_r, W_i; 
wire   [1:0]     W_ex_r, W_ex_i;
wire             scaling_bit = 1;
reg    [`W-1:0]  Min_r_reg,  Min_i_reg;
reg    [`W-1:0]  A_r_delay1,  A_i_delay1;
reg    [`W-1:0]  A_r_delay2,  A_i_delay2;
reg    [`W-1:0]  A_r_delay3,  A_i_delay3;
reg    [`W-1:0]  A_r_delay4,  A_i_delay4;
reg              k_delay1;
reg              k_delay2;
reg              k_delay3;
reg              k_delay4;
assign Min_r = (k[9] == 0) ? Fout_r : A_r;
assign Min_i = (k[9] == 0) ? Fout_i : A_i;
assign Fin_r = (k[9] == 0) ? X_r : B_r;
assign Fin_i = (k[9] == 0) ? X_i : B_i;
assign Mbf_r = (k_delay4 == 1) ? A_r_delay4 : Mout_r;
assign Mbf_i = (k_delay4 == 1) ? A_i_delay4 : Mout_i;
FIFO_ST4 FIFO_ST4(CLK, Fin_r, Fin_i, Fout_r, Fout_i);
TWIDDLE_ST4 TWIDDLE_ST4(CLK, k[8:0], W_ex_r, W_r, W_ex_i, W_i);
CADD CADD(Fout_r, Fout_i, X_r, X_i, A_r, A_i, B_r, B_i, scaling_bit);
CMUL CMUL(CLK, Min_r_reg, Min_i_reg, W_r, W_ex_r, W_i, W_ex_i, Mout_r, Mout_i);
always@(posedge CLK)
begin
    Min_r_reg <= Min_r;  Min_i_reg <= Min_i;
    k_delay1 <= k[9];
    k_delay2 <= k_delay1;
    k_delay3 <= k_delay2;
    k_delay4 <= k_delay3;
    A_r_delay1  <= A_r;         A_i_delay1 <= A_i;
    A_r_delay2  <= A_r_delay1;  A_i_delay2 <= A_i_delay1;
    A_r_delay3  <= A_r_delay2;  A_i_delay3 <= A_i_delay2;
    A_r_delay4  <= A_r_delay3;  A_i_delay4 <= A_i_delay3;
    Y_r <= Mbf_r;
    Y_i <= Mbf_i;
end
endmodule


module BUTTERFLY_STAGE5(CLK, k, X_r, X_i, Y_r, Y_i);
input  CLK;
input  [`N_AD_BUS-1:0]  k;
input  [`W-1:0]  X_r, X_i;
output [`W-1:0]  Y_r, Y_i;
reg    [`W-1:0]  Y_r, Y_i;
wire   [`W-1:0]  A_r, A_i, B_r, B_i;
wire   [`W-1:0]  Fin_r, Fin_i;
wire   [`W-1:0]  Fout_r, Fout_i;
wire   [`W-1:0]  Min_r,  Min_i;
wire   [`W-1:0]  Mbf_r, Mbf_i;
wire   [`W-1:0]  Mout_r, Mout_i;
wire   [`W-1:0]  W_r, W_i; 
wire   [1:0]     W_ex_r, W_ex_i;
wire             scaling_bit = 1;
reg    [`W-1:0]  Min_r_reg,  Min_i_reg;
reg    [`W-1:0]  A_r_delay1,  A_i_delay1;
reg    [`W-1:0]  A_r_delay2,  A_i_delay2;
reg    [`W-1:0]  A_r_delay3,  A_i_delay3;
reg    [`W-1:0]  A_r_delay4,  A_i_delay4;
reg              k_delay1;
reg              k_delay2;
reg              k_delay3;
reg              k_delay4;
assign Min_r = (k[8] == 0) ? Fout_r : A_r;
assign Min_i = (k[8] == 0) ? Fout_i : A_i;
assign Fin_r = (k[8] == 0) ? X_r : B_r;
assign Fin_i = (k[8] == 0) ? X_i : B_i;
assign Mbf_r = (k_delay4 == 1) ? A_r_delay4 : Mout_r;
assign Mbf_i = (k_delay4 == 1) ? A_i_delay4 : Mout_i;
FIFO_ST5 FIFO_ST5(CLK, Fin_r, Fin_i, Fout_r, Fout_i);
TWIDDLE_ST5 TWIDDLE_ST5(CLK, k[7:0], W_ex_r, W_r, W_ex_i, W_i);
CADD CADD(Fout_r, Fout_i, X_r, X_i, A_r, A_i, B_r, B_i, scaling_bit);
CMUL CMUL(CLK, Min_r_reg, Min_i_reg, W_r, W_ex_r, W_i, W_ex_i, Mout_r, Mout_i);
always@(posedge CLK)
begin
    Min_r_reg <= Min_r;  Min_i_reg <= Min_i;
    k_delay1 <= k[8];
    k_delay2 <= k_delay1;
    k_delay3 <= k_delay2;
    k_delay4 <= k_delay3;
    A_r_delay1  <= A_r;         A_i_delay1 <= A_i;
    A_r_delay2  <= A_r_delay1;  A_i_delay2 <= A_i_delay1;
    A_r_delay3  <= A_r_delay2;  A_i_delay3 <= A_i_delay2;
    A_r_delay4  <= A_r_delay3;  A_i_delay4 <= A_i_delay3;
    Y_r <= Mbf_r;
    Y_i <= Mbf_i;
end
endmodule


module BUTTERFLY_STAGE6(CLK, k, X_r, X_i, Y_r, Y_i);
input  CLK;
input  [`N_AD_BUS-1:0]  k;
input  [`W-1:0]  X_r, X_i;
output [`W-1:0]  Y_r, Y_i;
reg    [`W-1:0]  Y_r, Y_i;
wire   [`W-1:0]  A_r, A_i, B_r, B_i;
wire   [`W-1:0]  Fin_r, Fin_i;
wire   [`W-1:0]  Fout_r, Fout_i;
wire   [`W-1:0]  Min_r,  Min_i;
wire   [`W-1:0]  Mbf_r, Mbf_i;
wire   [`W-1:0]  Mout_r, Mout_i;
wire   [`W-1:0]  W_r, W_i; 
wire   [1:0]     W_ex_r, W_ex_i;
wire             scaling_bit = 1;
reg    [`W-1:0]  Min_r_reg,  Min_i_reg;
reg    [`W-1:0]  A_r_delay1,  A_i_delay1;
reg    [`W-1:0]  A_r_delay2,  A_i_delay2;
reg    [`W-1:0]  A_r_delay3,  A_i_delay3;
reg    [`W-1:0]  A_r_delay4,  A_i_delay4;
reg              k_delay1;
reg              k_delay2;
reg              k_delay3;
reg              k_delay4;
assign Min_r = (k[7] == 0) ? Fout_r : A_r;
assign Min_i = (k[7] == 0) ? Fout_i : A_i;
assign Fin_r = (k[7] == 0) ? X_r : B_r;
assign Fin_i = (k[7] == 0) ? X_i : B_i;
assign Mbf_r = (k_delay4 == 1) ? A_r_delay4 : Mout_r;
assign Mbf_i = (k_delay4 == 1) ? A_i_delay4 : Mout_i;
FIFO_ST6 FIFO_ST6(CLK, Fin_r, Fin_i, Fout_r, Fout_i);
TWIDDLE_ST6 TWIDDLE_ST6(CLK, k[6:0], W_ex_r, W_r, W_ex_i, W_i);
CADD CADD(Fout_r, Fout_i, X_r, X_i, A_r, A_i, B_r, B_i, scaling_bit);
CMUL CMUL(CLK, Min_r_reg, Min_i_reg, W_r, W_ex_r, W_i, W_ex_i, Mout_r, Mout_i);
always@(posedge CLK)
begin
    Min_r_reg <= Min_r;  Min_i_reg <= Min_i;
    k_delay1 <= k[7];
    k_delay2 <= k_delay1;
    k_delay3 <= k_delay2;
    k_delay4 <= k_delay3;
    A_r_delay1  <= A_r;         A_i_delay1 <= A_i;
    A_r_delay2  <= A_r_delay1;  A_i_delay2 <= A_i_delay1;
    A_r_delay3  <= A_r_delay2;  A_i_delay3 <= A_i_delay2;
    A_r_delay4  <= A_r_delay3;  A_i_delay4 <= A_i_delay3;
    Y_r <= Mbf_r;
    Y_i <= Mbf_i;
end
endmodule


module BUTTERFLY_STAGE7(CLK, k, X_r, X_i, Y_r, Y_i);
input  CLK;
input  [`N_AD_BUS-1:0]  k;
input  [`W-1:0]  X_r, X_i;
output [`W-1:0]  Y_r, Y_i;
reg    [`W-1:0]  Y_r, Y_i;
wire   [`W-1:0]  A_r, A_i, B_r, B_i;
wire   [`W-1:0]  Fin_r, Fin_i;
wire   [`W-1:0]  Fout_r, Fout_i;
wire   [`W-1:0]  Min_r,  Min_i;
wire   [`W-1:0]  Mbf_r, Mbf_i;
wire   [`W-1:0]  Mout_r, Mout_i;
wire   [`W-1:0]  W_r, W_i; 
wire   [1:0]     W_ex_r, W_ex_i;
wire             scaling_bit = 1;
reg    [`W-1:0]  Min_r_reg,  Min_i_reg;
reg    [`W-1:0]  A_r_delay1,  A_i_delay1;
reg    [`W-1:0]  A_r_delay2,  A_i_delay2;
reg    [`W-1:0]  A_r_delay3,  A_i_delay3;
reg    [`W-1:0]  A_r_delay4,  A_i_delay4;
reg              k_delay1;
reg              k_delay2;
reg              k_delay3;
reg              k_delay4;
assign Min_r = (k[6] == 0) ? Fout_r : A_r;
assign Min_i = (k[6] == 0) ? Fout_i : A_i;
assign Fin_r = (k[6] == 0) ? X_r : B_r;
assign Fin_i = (k[6] == 0) ? X_i : B_i;
assign Mbf_r = (k_delay4 == 1) ? A_r_delay4 : Mout_r;
assign Mbf_i = (k_delay4 == 1) ? A_i_delay4 : Mout_i;
FIFO_ST7 FIFO_ST7(CLK, Fin_r, Fin_i, Fout_r, Fout_i);
TWIDDLE_ST7 TWIDDLE_ST7(CLK, k[5:0], W_ex_r, W_r, W_ex_i, W_i);
CADD CADD(Fout_r, Fout_i, X_r, X_i, A_r, A_i, B_r, B_i, scaling_bit);
CMUL CMUL(CLK, Min_r_reg, Min_i_reg, W_r, W_ex_r, W_i, W_ex_i, Mout_r, Mout_i);
always@(posedge CLK)
begin
    Min_r_reg <= Min_r;  Min_i_reg <= Min_i;
    k_delay1 <= k[6];
    k_delay2 <= k_delay1;
    k_delay3 <= k_delay2;
    k_delay4 <= k_delay3;
    A_r_delay1  <= A_r;         A_i_delay1 <= A_i;
    A_r_delay2  <= A_r_delay1;  A_i_delay2 <= A_i_delay1;
    A_r_delay3  <= A_r_delay2;  A_i_delay3 <= A_i_delay2;
    A_r_delay4  <= A_r_delay3;  A_i_delay4 <= A_i_delay3;
    Y_r <= Mbf_r;
    Y_i <= Mbf_i;
end
endmodule


module BUTTERFLY_STAGE8(CLK, k, X_r, X_i, Y_r, Y_i);
input  CLK;
input  [`N_AD_BUS-1:0]  k;
input  [`W-1:0]  X_r, X_i;
output [`W-1:0]  Y_r, Y_i;
reg    [`W-1:0]  Y_r, Y_i;
wire   [`W-1:0]  A_r, A_i, B_r, B_i;
wire   [`W-1:0]  Fin_r, Fin_i;
wire   [`W-1:0]  Fout_r, Fout_i;
wire   [`W-1:0]  Min_r,  Min_i;
wire   [`W-1:0]  Mbf_r, Mbf_i;
wire   [`W-1:0]  Mout_r, Mout_i;
wire   [`W-1:0]  W_r, W_i; 
wire   [1:0]     W_ex_r, W_ex_i;
wire             scaling_bit = 1;
reg    [`W-1:0]  Min_r_reg,  Min_i_reg;
reg    [`W-1:0]  A_r_delay1,  A_i_delay1;
reg    [`W-1:0]  A_r_delay2,  A_i_delay2;
reg    [`W-1:0]  A_r_delay3,  A_i_delay3;
reg    [`W-1:0]  A_r_delay4,  A_i_delay4;
reg              k_delay1;
reg              k_delay2;
reg              k_delay3;
reg              k_delay4;
assign Min_r = (k[5] == 0) ? Fout_r : A_r;
assign Min_i = (k[5] == 0) ? Fout_i : A_i;
assign Fin_r = (k[5] == 0) ? X_r : B_r;
assign Fin_i = (k[5] == 0) ? X_i : B_i;
assign Mbf_r = (k_delay4 == 1) ? A_r_delay4 : Mout_r;
assign Mbf_i = (k_delay4 == 1) ? A_i_delay4 : Mout_i;
FIFO_ST8 FIFO_ST8(CLK, Fin_r, Fin_i, Fout_r, Fout_i);
TWIDDLE_ST8 TWIDDLE_ST8(CLK, k[4:0], W_ex_r, W_r, W_ex_i, W_i);
CADD CADD(Fout_r, Fout_i, X_r, X_i, A_r, A_i, B_r, B_i, scaling_bit);
CMUL CMUL(CLK, Min_r_reg, Min_i_reg, W_r, W_ex_r, W_i, W_ex_i, Mout_r, Mout_i);
always@(posedge CLK)
begin
    Min_r_reg <= Min_r;  Min_i_reg <= Min_i;
    k_delay1 <= k[5];
    k_delay2 <= k_delay1;
    k_delay3 <= k_delay2;
    k_delay4 <= k_delay3;
    A_r_delay1  <= A_r;         A_i_delay1 <= A_i;
    A_r_delay2  <= A_r_delay1;  A_i_delay2 <= A_i_delay1;
    A_r_delay3  <= A_r_delay2;  A_i_delay3 <= A_i_delay2;
    A_r_delay4  <= A_r_delay3;  A_i_delay4 <= A_i_delay3;
    Y_r <= Mbf_r;
    Y_i <= Mbf_i;
end
endmodule


module BUTTERFLY_STAGE9(CLK, k, X_r, X_i, Y_r, Y_i);
input  CLK;
input  [`N_AD_BUS-1:0]  k;
input  [`W-1:0]  X_r, X_i;
output [`W-1:0]  Y_r, Y_i;
reg    [`W-1:0]  Y_r, Y_i;
wire   [`W-1:0]  A_r, A_i, B_r, B_i;
wire   [`W-1:0]  Fin_r, Fin_i;
wire   [`W-1:0]  Fout_r, Fout_i;
wire   [`W-1:0]  Min_r,  Min_i;
wire   [`W-1:0]  Mbf_r, Mbf_i;
wire   [`W-1:0]  Mout_r, Mout_i;
wire   [`W-1:0]  W_r, W_i; 
wire   [1:0]     W_ex_r, W_ex_i;
wire             scaling_bit = 1;
reg    [`W-1:0]  Min_r_reg,  Min_i_reg;
reg    [`W-1:0]  A_r_delay1,  A_i_delay1;
reg    [`W-1:0]  A_r_delay2,  A_i_delay2;
reg    [`W-1:0]  A_r_delay3,  A_i_delay3;
reg    [`W-1:0]  A_r_delay4,  A_i_delay4;
reg              k_delay1;
reg              k_delay2;
reg              k_delay3;
reg              k_delay4;
assign Min_r = (k[4] == 0) ? Fout_r : A_r;
assign Min_i = (k[4] == 0) ? Fout_i : A_i;
assign Fin_r = (k[4] == 0) ? X_r : B_r;
assign Fin_i = (k[4] == 0) ? X_i : B_i;
assign Mbf_r = (k_delay4 == 1) ? A_r_delay4 : Mout_r;
assign Mbf_i = (k_delay4 == 1) ? A_i_delay4 : Mout_i;
FIFO_ST9 FIFO_ST9(CLK, Fin_r, Fin_i, Fout_r, Fout_i);
TWIDDLE_ST9 TWIDDLE_ST9(CLK, k[3:0], W_ex_r, W_r, W_ex_i, W_i);
CADD CADD(Fout_r, Fout_i, X_r, X_i, A_r, A_i, B_r, B_i, scaling_bit);
CMUL CMUL(CLK, Min_r_reg, Min_i_reg, W_r, W_ex_r, W_i, W_ex_i, Mout_r, Mout_i);
always@(posedge CLK)
begin
    Min_r_reg <= Min_r;  Min_i_reg <= Min_i;
    k_delay1 <= k[4];
    k_delay2 <= k_delay1;
    k_delay3 <= k_delay2;
    k_delay4 <= k_delay3;
    A_r_delay1  <= A_r;         A_i_delay1 <= A_i;
    A_r_delay2  <= A_r_delay1;  A_i_delay2 <= A_i_delay1;
    A_r_delay3  <= A_r_delay2;  A_i_delay3 <= A_i_delay2;
    A_r_delay4  <= A_r_delay3;  A_i_delay4 <= A_i_delay3;
    Y_r <= Mbf_r;
    Y_i <= Mbf_i;
end
endmodule


module BUTTERFLY_STAGE10(CLK, k, X_r, X_i, Y_r, Y_i);
input  CLK;
input  [`N_AD_BUS-1:0]  k;
input  [`W-1:0]  X_r, X_i;
output [`W-1:0]  Y_r, Y_i;
reg    [`W-1:0]  Y_r, Y_i;
wire   [`W-1:0]  A_r, A_i, B_r, B_i;
wire   [`W-1:0]  Fin_r, Fin_i;
wire   [`W-1:0]  Fout_r, Fout_i;
wire   [`W-1:0]  Min_r,  Min_i;
wire   [`W-1:0]  Mbf_r, Mbf_i;
wire   [`W-1:0]  Mout_r, Mout_i;
wire   [`W-1:0]  W_r, W_i; 
wire   [1:0]     W_ex_r, W_ex_i;
wire             scaling_bit = 1;
reg    [`W-1:0]  Min_r_reg,  Min_i_reg;
reg    [`W-1:0]  A_r_delay1,  A_i_delay1;
reg    [`W-1:0]  A_r_delay2,  A_i_delay2;
reg    [`W-1:0]  A_r_delay3,  A_i_delay3;
reg    [`W-1:0]  A_r_delay4,  A_i_delay4;
reg              k_delay1;
reg              k_delay2;
reg              k_delay3;
reg              k_delay4;
assign Min_r = (k[3] == 0) ? Fout_r : A_r;
assign Min_i = (k[3] == 0) ? Fout_i : A_i;
assign Fin_r = (k[3] == 0) ? X_r : B_r;
assign Fin_i = (k[3] == 0) ? X_i : B_i;
assign Mbf_r = (k_delay4 == 1) ? A_r_delay4 : Mout_r;
assign Mbf_i = (k_delay4 == 1) ? A_i_delay4 : Mout_i;
FIFO_ST10 FIFO_ST10(CLK, Fin_r, Fin_i, Fout_r, Fout_i);
TWIDDLE_ST10 TWIDDLE_ST10(CLK, k[2:0], W_ex_r, W_r, W_ex_i, W_i);
CADD CADD(Fout_r, Fout_i, X_r, X_i, A_r, A_i, B_r, B_i, scaling_bit);
CMUL CMUL(CLK, Min_r_reg, Min_i_reg, W_r, W_ex_r, W_i, W_ex_i, Mout_r, Mout_i);
always@(posedge CLK)
begin
    Min_r_reg <= Min_r;  Min_i_reg <= Min_i;
    k_delay1 <= k[3];
    k_delay2 <= k_delay1;
    k_delay3 <= k_delay2;
    k_delay4 <= k_delay3;
    A_r_delay1  <= A_r;         A_i_delay1 <= A_i;
    A_r_delay2  <= A_r_delay1;  A_i_delay2 <= A_i_delay1;
    A_r_delay3  <= A_r_delay2;  A_i_delay3 <= A_i_delay2;
    A_r_delay4  <= A_r_delay3;  A_i_delay4 <= A_i_delay3;
    Y_r <= Mbf_r;
    Y_i <= Mbf_i;
end
endmodule


module BUTTERFLY_STAGE11(CLK, k, X_r, X_i, Y_r, Y_i);
input  CLK;
input  [`N_AD_BUS-1:0]  k;
input  [`W-1:0]  X_r, X_i;
output [`W-1:0]  Y_r, Y_i;
reg    [`W-1:0]  Y_r, Y_i;
wire   [`W-1:0]  A_r, A_i, B_r, B_i;
wire   [`W-1:0]  Fin_r, Fin_i;
wire   [`W-1:0]  Fout_r, Fout_i;
wire   [`W-1:0]  Min_r,  Min_i;
wire   [`W-1:0]  Mbf_r, Mbf_i;
wire   [`W-1:0]  Mout_r, Mout_i;
wire   [`W-1:0]  W_r, W_i; 
wire   [1:0]     W_ex_r, W_ex_i;
wire             scaling_bit = 1;
reg    [`W-1:0]  Min_r_reg,  Min_i_reg;
reg    [`W-1:0]  A_r_delay1,  A_i_delay1;
reg    [`W-1:0]  A_r_delay2,  A_i_delay2;
reg    [`W-1:0]  A_r_delay3,  A_i_delay3;
reg    [`W-1:0]  A_r_delay4,  A_i_delay4;
reg              k_delay1;
reg              k_delay2;
reg              k_delay3;
reg              k_delay4;
assign Min_r = (k[2] == 0) ? Fout_r : A_r;
assign Min_i = (k[2] == 0) ? Fout_i : A_i;
assign Fin_r = (k[2] == 0) ? X_r : B_r;
assign Fin_i = (k[2] == 0) ? X_i : B_i;
assign Mbf_r = (k_delay4 == 1) ? A_r_delay4 : Mout_r;
assign Mbf_i = (k_delay4 == 1) ? A_i_delay4 : Mout_i;
FIFO_ST11 FIFO_ST11(CLK, Fin_r, Fin_i, Fout_r, Fout_i);
TWIDDLE_ST11 TWIDDLE_ST11(CLK, k[1:0], W_ex_r, W_r, W_ex_i, W_i);
CADD CADD(Fout_r, Fout_i, X_r, X_i, A_r, A_i, B_r, B_i, scaling_bit);
CMUL CMUL(CLK, Min_r_reg, Min_i_reg, W_r, W_ex_r, W_i, W_ex_i, Mout_r, Mout_i);
always@(posedge CLK)
begin
    Min_r_reg <= Min_r;  Min_i_reg <= Min_i;
    k_delay1 <= k[2];
    k_delay2 <= k_delay1;
    k_delay3 <= k_delay2;
    k_delay4 <= k_delay3;
    A_r_delay1  <= A_r;         A_i_delay1 <= A_i;
    A_r_delay2  <= A_r_delay1;  A_i_delay2 <= A_i_delay1;
    A_r_delay3  <= A_r_delay2;  A_i_delay3 <= A_i_delay2;
    A_r_delay4  <= A_r_delay3;  A_i_delay4 <= A_i_delay3;
    Y_r <= Mbf_r;
    Y_i <= Mbf_i;
end
endmodule


module BUTTERFLY_STAGE12(CLK, k, X_r, X_i, Y_r, Y_i);
input  CLK;
input  [`N_AD_BUS-1:0]  k;
input  [`W-1:0]  X_r, X_i;
output [`W-1:0]  Y_r, Y_i;
reg    [`W-1:0]  Y_r, Y_i;
wire   [`W-1:0]  A_r, A_i, B_r, B_i;
wire   [`W-1:0]  Fin_r, Fin_i;
wire   [`W-1:0]  Fout_r, Fout_i;
wire   [`W-1:0]  Min_r,  Min_i;
wire   [`W-1:0]  Mbf_r, Mbf_i;
wire   [`W-1:0]  Mout_r, Mout_i;
wire   [`W-1:0]  W_r, W_i; 
wire   [1:0]     W_ex_r, W_ex_i;
wire             scaling_bit = 1;
reg    [`W-1:0]  Min_r_reg,  Min_i_reg;
reg    [`W-1:0]  A_r_delay1,  A_i_delay1;
reg    [`W-1:0]  A_r_delay2,  A_i_delay2;
reg    [`W-1:0]  A_r_delay3,  A_i_delay3;
reg    [`W-1:0]  A_r_delay4,  A_i_delay4;
reg              k_delay1;
reg              k_delay2;
reg              k_delay3;
reg              k_delay4;
assign Min_r = (k[1] == 0) ? Fout_r : A_r;
assign Min_i = (k[1] == 0) ? Fout_i : A_i;
assign Fin_r = (k[1] == 0) ? X_r : B_r;
assign Fin_i = (k[1] == 0) ? X_i : B_i;
assign Mbf_r = (k_delay4 == 1) ? A_r_delay4 : Mout_r;
assign Mbf_i = (k_delay4 == 1) ? A_i_delay4 : Mout_i;
FIFO_ST12 FIFO_ST12(CLK, Fin_r, Fin_i, Fout_r, Fout_i);
TWIDDLE_ST12 TWIDDLE_ST12(CLK, k[0], W_ex_r, W_r, W_ex_i, W_i);
CADD CADD(Fout_r, Fout_i, X_r, X_i, A_r, A_i, B_r, B_i, scaling_bit);
CMUL CMUL(CLK, Min_r_reg, Min_i_reg, W_r, W_ex_r, W_i, W_ex_i, Mout_r, Mout_i);
always@(posedge CLK)
begin
    Min_r_reg <= Min_r;  Min_i_reg <= Min_i;
    k_delay1 <= k[1];
    k_delay2 <= k_delay1;
    k_delay3 <= k_delay2;
    k_delay4 <= k_delay3;
    A_r_delay1  <= A_r;         A_i_delay1 <= A_i;
    A_r_delay2  <= A_r_delay1;  A_i_delay2 <= A_i_delay1;
    A_r_delay3  <= A_r_delay2;  A_i_delay3 <= A_i_delay2;
    A_r_delay4  <= A_r_delay3;  A_i_delay4 <= A_i_delay3;
    Y_r <= Mbf_r;
    Y_i <= Mbf_i;
end
endmodule


module BUTTERFLY_STAGE13(CLK, k, X_r, X_i, Y_r, Y_i);
input  CLK;
input  [`N_AD_BUS-1:0]  k;
input  [`W-1:0]  X_r, X_i;
output [`W-1:0]  Y_r, Y_i;
reg    [`W-1:0]  Y_r, Y_i;
wire   [`W-1:0]  A_r, A_i, B_r, B_i;
wire   [`W-1:0]  Fin_r, Fin_i;
wire   [`W-1:0]  Fout_r, Fout_i;
wire   [`W-1:0]  Min_r,  Min_i;
wire   [`W-1:0]  Mbf_r, Mbf_i;
wire			 scaling_bit = 0;
assign Min_r = (k[0] == 0) ? Fout_r : A_r;
assign Min_i = (k[0] == 0) ? Fout_i : A_i;
assign Fin_r = (k[0] == 0) ? X_r : B_r;
assign Fin_i = (k[0] == 0) ? X_i : B_i;
assign Mbf_r = Min_r;
assign Mbf_i = Min_i;
FIFO_ST13 FIFO_ST13(CLK, Fin_r, Fin_i, Fout_r, Fout_i);
CADD CADD(Fout_r, Fout_i, X_r, X_i, A_r, A_i, B_r, B_i, scaling_bit);
always@(posedge CLK)
begin
    Y_r <= Mbf_r;
    Y_i <= Mbf_i;
end
endmodule

module CADD(X_r, X_i, Y_r, Y_i, A_r, A_i, B_r, B_i, scaling_bit);
input  signed [`W-1:0]  X_r, X_i, Y_r, Y_i;
output signed [`W-1:0]  A_r, A_i, B_r, B_i;
input  scaling_bit;
wire   signed [`W:0]  m_r, m_i, n_r, n_i;
assign m_r = X_r + Y_r;
assign m_i = X_i + Y_i;
assign n_r = X_r - Y_r;
assign n_i = X_i - Y_i;
assign A_r = (scaling_bit == 1) ? m_r >>> 1 : m_r;
assign A_i = (scaling_bit == 1) ? m_i >>> 1 : m_i;
assign B_r = (scaling_bit == 1) ? n_r >>> 1 : n_r;
assign B_i = (scaling_bit == 1) ? n_i >>> 1 : n_i;
endmodule
module CMUL(CLK, X_r, X_i, W_r, W_ex_r, W_i, W_ex_i, Y_r, Y_i);
input  CLK;
input  [`W-1:0]  X_r, X_i;
input  [`W-1:0]  W_r, W_i;
input  [1:0]   W_ex_r, W_ex_i;
output [`W-1:0]  Y_r, Y_i;
wire   [2*`W-2:0]  m_rr, m_ri, m_ir, m_ii;
reg    signed [2*`W-2:0]  n_rr, n_ri, n_ir, n_ii;
wire   [2*`W-1:0]  o_r, o_i;
wire   signed [2*`W-2:0]  a_r, a_i;
wire 	zero = 0;
reg    signed [2*`W-2:0]  a_r_delay1, a_i_delay1;
reg    signed [2*`W-2:0]  a_r_delay2, a_i_delay2;
reg    [1:0]   W_ex_r_delay1, W_ex_i_delay1;
reg    [1:0]   W_ex_r_delay2, W_ex_i_delay2;
MUL MUL1(CLK, W_r, X_r, m_rr);
MUL MUL2(CLK, W_i, X_r, m_ri);
MUL MUL3(CLK, W_r, X_i, m_ir);
MUL MUL4(CLK, W_i, X_i, m_ii);
assign a_r  = {X_r, {`W-1{zero}}};
assign a_i  = {X_i, {`W-1{zero}}};
assign o_r = n_rr - n_ii;
assign o_i = n_ir + n_ri;
assign Y_r = o_r[2*`W-2:`W-1];
assign Y_i = o_i[2*`W-2:`W-1];
always@(posedge CLK)
begin
      a_r_delay1<=a_r;              a_i_delay1<=a_i;
      a_r_delay2<=a_r_delay1;       a_i_delay2<=a_i_delay1;
      W_ex_r_delay1<=W_ex_r;        W_ex_i_delay1<=W_ex_i;
      W_ex_r_delay2<=W_ex_r_delay1; W_ex_i_delay2<=W_ex_i_delay1;
      n_rr <= (W_ex_r_delay2 == 1) ?   a_r_delay2 :
              (W_ex_r_delay2 == 2) ?  -a_r_delay2 :
                                       m_rr;
      n_ri <= (W_ex_i_delay2 == 1) ?   a_r_delay2 :
              (W_ex_i_delay2 == 2) ?  -a_r_delay2 :
                                       m_ri;
      n_ir <= (W_ex_r_delay2 == 1) ?   a_i_delay2 :
              (W_ex_r_delay2 == 2) ?  -a_i_delay2 :
                                       m_ir;
      n_ii <= (W_ex_i_delay2 == 1) ?   a_i_delay2 :
              (W_ex_i_delay2 == 2) ?  -a_i_delay2 :
                                       m_ii;
end
endmodule
