////                                                             
////  Pipeline FFT Processor                                     
////                                                             
////  Author: Shingo Yoshizawa                                   
////          yoshizawa@islab.elec.kitami-it.ac.jp               
////                                                             
////  Source Files:                                              
////          definition.v                                       
////          FFT_SIM.v                                          
////          FFT.v                                              
////          FFT_BUTTERFLY.v                                    
////          FFT_FIFO.v                                         
////          FFT_TWIDDLE.v                                      
////                                                             
//// Copyright (C) 2013 Shingo Yoshizawa                         
////                    yoshizawa@islab.elec.kitami-it.ac.jp     
////                                                             
//// These source files may be used and distributed without      
//// restriction provided that this copyright statement is not   
//// removed from the files and that any derivative work contains
//// the original copyright notice and the associated disclaimer.
////                                                             
////     THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY     
//// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED   
//// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS   
//// FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE AUTHOR      
//// OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,         
//// INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES    
//// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE   
//// GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR        
//// BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF  
//// LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT  
//// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  
//// OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE         
//// POSSIBILITY OF SUCH DAMAGE.                                 
////                                                             


`timescale 1ns/10ps

`ifdef SYNTOOL_OFF 
`else 
`include "definition.v" 
`endif 

module FFT_SIM;

parameter T=10;

reg          CLK;
reg          RESET;
reg          EN;

reg  signed [`W-1:0]  X_r, X_i; // Input Data
wire signed [`W-1:0]  Y_r, Y_i; // Output Data

reg  signed [`W-1:0]  Y_r_ans, Y_i_ans; // Expected Data

reg  [`SIM_ITER_AD_BUS:0]            s, s1;
reg  [`N_AD_BUS:0]                   j, j1;
reg  [`N_AD_BUS+`SIM_ITER_AD_BUS:0]  err_num;
reg			 err_status;

integer      m1,m2,m3,m4,m5;

FFT FFT(CLK, RESET, EN, X_r, X_i, Y_r, Y_i);

 always #(T/2) CLK <= ~CLK;

initial
begin
   $timeformat(-9, 2, "ns", 8);
   
   CLK=0; RESET=0; EN=0;
   #(T/2)
   RESET = 1;
   #(T*2)
   EN = 1;
   
   m1 = $fopen("fft_input.txt","r");
   for (s=1; s <=`SIM_NUM_ITER; s=s+1)
	 begin
	    for (j=0; j <=`N-1; j=j+1)
	     begin
	      #T
	      m2 = $fscanf(m1,"%d %d\n", X_r, X_i);
	     end
     end
   $fclose(m1);

   #(T*`N*2)
   EN = 0;
   $finish;
end

initial
begin
	err_num = 0;
   	m3 = $fopen("fft_output.txt","r");
   	m5 = $fopen("fft_simulation_log.txt","w");
	#(T/2+T*(2+`DELAY_CYCLE+`N))
   	for (s1=1; s1 <=`SIM_NUM_ITER; s1=s1+1)
	 begin
	    for (j1=0; j1 <=`N-1; j1=j1+1)
	     begin
	      #T
	      m4 = $fscanf(m3,"%d %d\n", Y_r_ans, Y_i_ans);
	      if ((Y_r==Y_r_ans)&&(Y_i==Y_i_ans))
	      	  begin
	      	    err_status = 0;
	      	  end
	      else
	      	  begin
	      	    err_status =  1;
	      	    err_num = err_num + 1;
	      	  end
	      $display("Iter %d Index %d : Y_r %d : Y_r_ans %d : Y_i %d : Y_i_ans %d : Error %d ", s1, j1, Y_r, Y_r_ans, Y_i, Y_i_ans, err_status);
	      $fdisplay(m5,"Iter %d Index %d : Y_r %d : Y_r_ans %d : Y_i %d : Y_i_ans %d : Error %d ", s1, j1, Y_r, Y_r_ans, Y_i, Y_i_ans, err_status);
	     end
     end
     $fclose(m3);
     $fclose(m5);
	 $display("Total Error Count : %d", err_num);
end

  initial begin
    $dumpfile("FFT.vcd");
    $dumpvars(0, FFT);
  end
  
endmodule

